# Generated by Django 2.2.26 on 2022-02-10 17:13

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0015_dailyactivity"),
    ]

    operations = [
        migrations.CreateModel(
            name="JoinReleaseStudent",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("action_type", models.CharField(max_length=64)),
                (
                    "action_time",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                (
                    "student",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="student",
                        to="common.Student",
                    ),
                ),
            ],
        ),
    ]
