# Generated by Django 3.2.12 on 2022-04-07 16:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0019_aimmocharacter_alt"),
    ]

    operations = [
        migrations.AddField(
            model_name="class",
            name="is_active",
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name="class",
            name="access_code",
            field=models.CharField(max_length=5, null=True),
        ),
    ]
